package hk.advanpro.android.core.utils;


import org.apache.commons.lang3.time.DateFormatUtils;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;
import java.util.concurrent.TimeUnit;

/**
 * Created by jalin on 2017/12/7.
 */

public class DateUtils {
    private static final int YESTERDY = -1;
    private static final int TODAY = 0;
    private static final int TOMORROWDAT = 1;


    public static final String formatUTCString = "yyyy-MM-dd'T'HH:mm:ss.SSSZ";

    /**
     * 获取当天的小时
     *
     * @return 结果
     */
    public static int getHour() {
        Calendar c = Calendar.getInstance();//可以对每个时间域单独修改
        return c.get(Calendar.HOUR_OF_DAY);

    }

    /**
     * 判断两个日期是否在同一类型的范围内
     *
     * @param field 给定的日历字段。Calendar.DATE, Calendar.MONTH, Calendar.YEAR
     * @param date1 日期
     * @param date2 日期
     * @return 结果
     */
    public static boolean isSame(int field, long date1, long date2) {
        Calendar cal1 = Calendar.getInstance();
        cal1.setTimeInMillis(date1);
        Calendar cal2 = Calendar.getInstance();
        cal2.setTimeInMillis(date2);
        return isSame(field, cal1, cal2);
    }

    /**
     * 判断两个日期是否在同一类型的范围内
     *
     * @param field 给定的日历字段。Calendar.DATE, Calendar.MONTH, Calendar.YEAR
     * @param date1 日期
     * @param date2 日期
     * @return 结果
     */
    public static boolean isSame(int field, Date date1, Date date2) {
        if (date1 == null || date2 == null) {
            return false;
        }
        Calendar cal1 = Calendar.getInstance();
        cal1.setTime(date1);
        Calendar cal2 = Calendar.getInstance();
        cal2.setTime(date2);
        return isSame(field, cal1, cal2);
    }

    /**
     * 判断两个日期是否在同一类型的范围内
     *
     * @param field 给定的日历字段。Calendar.DATE, Calendar.MONTH, Calendar.YEAR,Calendar.WEEK_OF_YEAR
     * @param date1 日期
     * @param date2 日期
     * @return 结果
     */
    public static boolean isSame(int field, Calendar date1, Calendar date2) {
        if (date1 == null || date2 == null) {
            return false;
        }
        switch (field) {
            case Calendar.DATE:
                return date1.get(Calendar.ERA) == date2.get(Calendar.ERA) &&
                        date1.get(Calendar.YEAR) == date2.get(Calendar.YEAR) &&
                        date1.get(Calendar.DAY_OF_YEAR) == date2.get(Calendar.DAY_OF_YEAR);
            case Calendar.MONTH:
                return date1.get(Calendar.ERA) == date2.get(Calendar.ERA) &&
                        date1.get(Calendar.YEAR) == date2.get(Calendar.YEAR) &&
                        date1.get(Calendar.MONTH) == date2.get(Calendar.MONTH);
            case Calendar.YEAR:
                return date1.get(Calendar.ERA) == date2.get(Calendar.ERA) &&
                        date1.get(Calendar.YEAR) == date2.get(Calendar.YEAR);
            case Calendar.WEEK_OF_YEAR:
                return date1.get(Calendar.WEEK_OF_YEAR) == date2.get(Calendar.WEEK_OF_YEAR);
            default:
                return false;
        }
    }

    /**
     * 得到UTC时间，类型为字符串，格式为"yyyy-MM-dd'T'HH:mm:ss.SSS'Z'"
     *
     * @param date 日期
     * @return 得到UTC时间
     */
    public static String formatDateToUTC(Date date) {
        return DateFormatUtils.formatUTC(date, formatUTCString);
    }

    /**
     * 得到UTC时间，类型为字符串，格式为"yyyy-MM-dd'T'HH:mm:ss.SSS'Z'"
     *
     * @param date 日期
     * @return 得到UTC时间
     */
    public static String formatDateToUTC(long date) {
        return DateFormatUtils.formatUTC(date, formatUTCString);
    }

    /**
     * 将此日期时间设置成0点整
     *
     * @param date 日期
     * @return 时间设置成0点整
     */
    public static Calendar getStartOfDay(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar;
    }

    /**
     * 将此日期时间设置成23:999
     *
     * @param date 日期
     * @return 时间设置成23:999
     */
    public static Calendar getEndOfDay(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        return calendar;
    }

    /**
     * 获取日期所在月份的的第一天
     */
    public static Date getStartOfMonth(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.DATE, 1);
        return calendar.getTime();
    }

    /**
     * 获取日期所在月份的的最后一天
     */
    public static Date getEndOfMonth(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.DATE, calendar.getActualMaximum(Calendar.DATE));
        return calendar.getTime();
    }

    /**
     * 获取日期所在年的最后一天
     */
    public static Date getEndOfYear(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.MONTH, 11);
        calendar.set(Calendar.DATE, calendar.getActualMaximum(Calendar.DATE));
        return calendar.getTime();
    }

    /**
     * 获取日期所在年的第一天
     */
    public static Date getStartOfYear(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.MONTH, 0);
        calendar.set(Calendar.DATE, 1);
        return calendar.getTime();
    }

    /**
     * 获取日子所在的周一.
     *
     * @param date
     * @return
     */
    public static Date getStartOfWeekMonday(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(calendar.DAY_OF_WEEK, calendar.MONDAY);
        return calendar.getTime();
    }

    /**
     * 获取日子所在的周日.
     *
     * @param date
     * @return
     */
    public static Date getStartOfWeekSaturday(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.set(calendar.DAY_OF_WEEK, calendar.SATURDAY);
        calendar.setTime(date);
        return calendar.getTime();
    }


    /**
     * 将此日期时间设置成0点整
     *
     * @param date 日期
     * @return 时间设置成0点整
     */
    public static long getStartOfDay(long date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date(date));
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar.getTimeInMillis();
    }

    /**
     * 返回指定格式日期
     *
     * @param date    日期
     * @param pattern 解析
     * @return 字符串
     */
    public static String formatDate(Date date, String pattern) {
        return new SimpleDateFormat(pattern).format(date);
    }

    /**
     * 返回指定格式日期
     *
     * @param time    日期
     * @param pattern 解析
     * @return 字符串
     */
    public static String formatDate(long time, String pattern) {
        return new SimpleDateFormat(pattern).format(time);
    }


    /**
     * 获取后一天日期
     *
     * @param date 日期
     * @return 后一天日期
     */
    public static long getNextDay(long date) {
        return getDay(date, 1);
    }

    /**
     * 获取基于当前日期偏移后的日期
     *
     * @param date   当前日期
     * @param offset 偏移的天数，负数向前偏移，正数向后偏移
     * @return 结果
     */
    public static Date getDay(Date date, int offset) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.DATE, offset);
        return calendar.getTime();
    }

    /**
     * 获取基于当前日期偏移后的日期
     *
     * @param date   当前日期
     * @param offset 偏移的天数，负数向前偏移，正数向后偏移
     * @return 结果
     */
    public static long getDay(long date, int offset) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(date);
        calendar.add(Calendar.DATE, offset);
        return calendar.getTimeInMillis();
    }

    /**
     * 两个日期相差的天数(B-A)
     */
    public static int daysBetween(long dateA, long dateB) {
        dateA = getStartOfDay(dateA);
        dateB = getStartOfDay(dateB);
        return (int) ((dateB - dateA) / (24 * 3600000L));
    }

    /**
     * 两个日期相差的月数(B-A)
     */
    public static int monthsBetween(Date dateA, Date dateB) {
        Calendar calendarA = Calendar.getInstance();
        Calendar calendarB = Calendar.getInstance();
        calendarA.setTime(dateA);
        calendarB.setTime(dateB);
        return (calendarB.get(Calendar.YEAR) - calendarA.get(Calendar.YEAR)) * 12 +
                (calendarB.get(Calendar.MONTH) - calendarA.get(Calendar.MONTH));
    }

    /**
     * 获取当前日期对应是星期几
     *
     * @param date 当前日期
     * @return 与Calendar的星期字段相同，1表示星期天，以此类推
     */
    public static int getDayOfWeek(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        return calendar.get(Calendar.DAY_OF_WEEK);
    }

    /**
     * 获取指定日期的当天毫秒数
     */
    public static long getMillisInDay(Date date) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        return c.get(Calendar.HOUR_OF_DAY) * 3600000 + c.get(Calendar.MINUTE) * 60000 +
                c.get(Calendar.SECOND) * 1000 + c.get(Calendar.MILLISECOND);
    }

    /**
     * 根据条件获取相应最大值，如此日期的月有多少天，Calendar.DATE
     */
    public static int getActualMaximum(Date date, int field) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        return calendar.getActualMaximum(field);
    }


    //判断选择的日期是否是本月
    public static boolean isThisMonth(long time) {
        return isThisTime(time, "yyyy-MM");
    }

    private static boolean isThisTime(long time, String pattern) {
        Date date = new Date(time);
        SimpleDateFormat sdf = new SimpleDateFormat(pattern);
        String param = sdf.format(date);//参数时间
        String now = sdf.format(new Date());//当前时间
        if (param.equals(now)) {
            return true;
        }
        return false;
    }

    public static boolean isUpMonth(Date time) {
        Calendar cal = Calendar.getInstance();
        int i = cal.get(Calendar.MONTH);
        cal.setTime(time);
        int i1 = cal.get(Calendar.MONTH) + 1;
        return i == i1;
    }

    /**
     * 由出生日期获得年龄
     *
     * @param birthDay 出生日期
     * @return 年龄
     */
    public static int getAge(Date birthDay) {
        Calendar cal = Calendar.getInstance();

        if (cal.before(birthDay)) {
            throw new IllegalArgumentException(
                    "The birthDay is before Now.It's unbelievable!");
        }
        int yearNow = cal.get(Calendar.YEAR);
        int monthNow = cal.get(Calendar.MONTH);
        int dayOfMonthNow = cal.get(Calendar.DAY_OF_MONTH);
        cal.setTime(birthDay);

        int yearBirth = cal.get(Calendar.YEAR);
        int monthBirth = cal.get(Calendar.MONTH);
        int dayOfMonthBirth = cal.get(Calendar.DAY_OF_MONTH);

        int age = yearNow - yearBirth;

        if (monthNow <= monthBirth) {
            if (monthNow == monthBirth) {
                if (dayOfMonthNow < dayOfMonthBirth) age--;
            } else {
                age--;
            }
        }
        return age;
    }

    /**
     * 获取当前时区
     *
     * @return
     */
    public static long getCurrentTimeZone() {
        TimeZone mTimeZone = TimeZone.getDefault();
        int mGMTOffset = mTimeZone.getRawOffset();
        long tz = TimeUnit.HOURS.convert(mGMTOffset, TimeUnit.MILLISECONDS);
        return tz;
    }

    /**
     * 判断当前系统时间是否在指定时间的范围内
     *
     * @param timeA
     * @param timeB
     * @return
     */
    public static boolean isCurrentInTimeScope(long timeA, long timeB) {
        return System.currentTimeMillis() > timeA && System.currentTimeMillis() < timeB;
    }

    /***
     * 获取指定一天中指定的时间的时间戳
     *
     * @param hour
     * @return
     */
    public static long getSpeDate(int hour) {
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.HOUR_OF_DAY, hour);
        cal.set(Calendar.SECOND, 0);
        cal.set(Calendar.MINUTE, 0);
        cal.set(Calendar.MILLISECOND, 0);

        return cal.getTimeInMillis();
    }

}
