package hk.advanpro.android.core.utils;

import android.app.DownloadManager;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.database.Cursor;
import android.net.ConnectivityManager;
import android.net.Uri;
import android.os.Environment;
import android.util.LongSparseArray;

import java.io.File;
import java.io.FileInputStream;
import java.security.MessageDigest;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by jalin on 2018/2/26.
 */

public class DownloadUtils {
    /**
     * 创建
     *
     * @param context
     * @return
     */
    public static DownloadUtils create(Context context) {
        instance = new DownloadUtils(context);
        return instance;
    }

    /**
     * 退出时释放资源
     */
    public static void destroy() {
        instance.mContext.unregisterReceiver(instance.receiver);
    }

    public static DownloadUtils getInstance() {
        return instance;
    }

    /**
     * 下载
     */
    public void startDownload(DownloadInfo downloadInfo, DownloadCallback callback) {
        List<DownloadCallback> callbacks = downloadCallback.get(downloadInfo.url);
        if (callbacks == null) {
            callbacks = new ArrayList<>();
            downloadCallback.put(downloadInfo.url, callbacks);
        }
        callbacks.add(callback);
        if (isDownloading(downloadInfo.url)) {
            // 正在下载不用重复下载
            return;
        }
        String fileName = org.apache.commons.lang3.StringUtils.substring(downloadInfo.url, org.apache.commons.lang3.StringUtils.lastIndexOf(downloadInfo.url, "/") + 1);
        File targetFile = new File(Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS), fileName);
        downloadInfo.targetFile = targetFile;

        if (targetFile.exists() && downloadInfo.md5 != null && downloadInfo.md5.equalsIgnoreCase(getFileMd5Code(targetFile))) {
            // 文件已存在，并且MD5码正确,文件已下载，不用重复下载
            callback.onSuccess(downloadInfo);
            return;
        }
        if (targetFile.exists()) {
            targetFile.delete();
        }
        //创建下载任务
        DownloadManager.Request request = new DownloadManager.Request(Uri.parse(downloadInfo.url));

        //在通知栏中显示，默认不显示
        request.setNotificationVisibility(DownloadManager.Request.VISIBILITY_VISIBLE);
        if (downloadInfo.isNotification) {
            request.setTitle(downloadInfo.title);
            request.setDescription(downloadInfo.description);
            request.setVisibleInDownloadsUi(true);
        } else {
            request.setVisibleInDownloadsUi(false);
        }

        // 允许wifi环境，移动网络环境下载
        request.setAllowedNetworkTypes(DownloadManager.Request.NETWORK_MOBILE | DownloadManager.Request.NETWORK_WIFI);
        // 移动网络情况下是否允许漫游
//        request.setAllowedOverRoaming(false);
        // 设置下载的路径
        request.setDestinationUri(Uri.fromFile(targetFile));

        // 将下载请求加入下载队列，加入下载队列后会给该任务返回一个long型的id，通过该id可以取消任务，重启任务、获取下载的文件等等
        downloadInfo.downloadId = downloadManager.enqueue(request);

        downloadInfos.put(downloadInfo.downloadId, downloadInfo);
    }

    public boolean isDownloading(String url) {
        for (int i = 0; i < instance.downloadInfos.size(); i++) {
            if (instance.downloadInfos.valueAt(i).url.equals(url)) {
                return true;
            }
        }
        return false;
    }


    public static class DownloadInfo {
        public long downloadId;// 下载的ID
        public String url; // 下载URL
        public File targetFile;//下载到的路径
        public String md5;
        public boolean isNotification; // 是否打开下载通知进度条
        public String title; // 下载通知进度条标题
        public String description; // 下载通知进度条内容
    }

    public interface DownloadCallback {
        void onSuccess(DownloadInfo info);

        void onError(DownloadInfo info, int errorCode);
    }

    private DownloadUtils(Context context) {
        this.mContext = context;
        //获取DownloadManager
        downloadManager = (DownloadManager) mContext.getSystemService(Context.DOWNLOAD_SERVICE);

        //注册广播接收者，监听下载状态
        IntentFilter intentFilter = new IntentFilter();
        intentFilter.addAction(DownloadManager.ACTION_DOWNLOAD_COMPLETE);
        intentFilter.addAction(ConnectivityManager.CONNECTIVITY_ACTION);
        mContext.registerReceiver(receiver, intentFilter);
    }

    private static DownloadUtils instance;

    private LongSparseArray<DownloadInfo> downloadInfos = new LongSparseArray<>();
    private Map<String, List<DownloadCallback>> downloadCallback = new HashMap<>();
    //下载器
    private DownloadManager downloadManager;
    //上下文
    private Context mContext;

    //广播监听下载的各个状态
    private BroadcastReceiver receiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            if (DownloadManager.ACTION_DOWNLOAD_COMPLETE.equals(intent.getAction())) {
                // 下载状态改变
                long downloadId = intent.getLongExtra(DownloadManager.EXTRA_DOWNLOAD_ID, -1);
                DownloadManager.Query query = new DownloadManager.Query();
                //通过下载的id查找
                query.setFilterById(downloadId);
                DownloadInfo downloadInfo = downloadInfos.get(downloadId);
                if (downloadInfo == null)
                    return;
                Cursor c = instance.downloadManager.query(query);

                if (c.moveToFirst()) {
                    int status = c.getInt(c.getColumnIndex(DownloadManager.COLUMN_STATUS));
                    switch (status) {
                        //下载暂停
                        case DownloadManager.STATUS_PAUSED:
                            break;
                        //下载延迟
                        case DownloadManager.STATUS_PENDING:
                            break;
                        //正在下载
                        case DownloadManager.STATUS_RUNNING:
                            break;
                        //下载完成
                        case DownloadManager.STATUS_SUCCESSFUL: {
                            List<DownloadCallback> downloadCallbacks = downloadCallback.get(downloadInfo.url);
                            if (!downloadInfo.targetFile.exists() || downloadInfo.md5 != null && !downloadInfo.md5.equalsIgnoreCase(getFileMd5Code(downloadInfo.targetFile))) {
                                // 文件MD5校验失败
                                if (downloadInfo.targetFile.exists())
                                    downloadInfo.targetFile.delete();
                                if (downloadCallbacks != null) {
                                    for (DownloadCallback callback : downloadCallbacks) {
                                        callback.onError(downloadInfo, 0);
                                    }
                                }
                            } else {
                                // 文件下载成功
                                if (downloadCallbacks != null) {
                                    for (DownloadCallback callback : downloadCallbacks) {
                                        callback.onSuccess(downloadInfo);
                                    }
                                }
                            }
                            downloadInfos.remove(downloadId);//移出任务集合
                            downloadCallback.remove(downloadInfo.url);
                            break;
                        }
                        case DownloadManager.STATUS_FAILED: {
                            //下载失败
                            List<DownloadCallback> downloadCallbacks = downloadCallback.get(downloadInfo.url);
                            int reason = c.getInt(c.getColumnIndex(DownloadManager.COLUMN_REASON));
                            if (downloadInfo.targetFile.exists())
                                downloadInfo.targetFile.delete();
                            if (downloadCallbacks != null) {
                                for (DownloadCallback callback : downloadCallbacks) {
                                    callback.onError(downloadInfo, reason);
                                }
                            }
                            downloadInfos.remove(downloadId);//移出任务集合
                            downloadCallback.remove(downloadInfo.url);
                            break;

                        }
                    }
                }
            } else if (ConnectivityManager.CONNECTIVITY_ACTION.equals(intent.getAction())) {
                // 网络状态改变
                if (!SystemUtils.isCurrentNetworkWIFI(context)) {
                    // 非wifi状态

                } else {
                    // wifi状态

                }
            }
        }
    };

    /**
     * 获取文件的md5值
     *
     * @param file 文件
     * @return md5值，文件不存在返回null
     */
    public static String getFileMd5Code(File file) {
        try {
            FileInputStream in = new FileInputStream(file);
            MessageDigest messageDigest = MessageDigest.getInstance("md5");
            byte[] buf = new byte[1024];
            int len;
            while ((len = in.read(buf)) != -1) {
                messageDigest.update(buf, 0, len);
            }
            in.close();
            return MD5Utils.bytesToString(messageDigest.digest());
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }


    //下载到本地后执行安装
    public void installAPK(long downloadId) {
        //获取下载文件的Uri
        Uri downloadFileUri = downloadManager.getUriForDownloadedFile(downloadId);
        if (downloadFileUri != null) {
            Intent intent = new Intent(Intent.ACTION_VIEW);
            intent.setDataAndType(downloadFileUri, "application/vnd.android.package-archive");
            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            mContext.startActivity(intent);
        }
    }
}