package hk.advanpro.android.core.utils;

import android.text.TextUtils;

import org.apache.commons.lang3.StringUtils;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

/**
 * Created by jalin on 2017/12/6.
 */

public class FileUtils {
    private String _filePath;
    private File _file;

    public boolean write(String data, String path) {
        if (!StringUtils.isNotEmpty(data) || !StringUtils.isNotEmpty(path))
            return false;

        if (!path.equals(_filePath)) {
            //上次写入的路径为null或者不一致
            this._filePath = path;

            String fileName = getFileNameAndType(path);
            String filePath = getFilePath(path);

            if (!StringUtils.isNotEmpty(fileName) || !StringUtils.isNotEmpty(filePath)) {
                // 文件名为空,文件路径为空直接返回false
                return false;
            }
            //先创建文件
            File parentFile = new File(filePath);
            if (!parentFile.exists()) {
                //文件目录不存在先创建目录
                parentFile.mkdirs();
            }


            //实例化文件
            File file = new File(parentFile, fileName);
            _file = file;
        }

        //写入文件
        return write(_file, data);
    }

    public static boolean write(File file, String data) {
        FileOutputStream os = null;
        try {
            os = new FileOutputStream(file, true);
            os.write(data.getBytes());
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        } finally {
            if (os != null) {
                try {
                    os.close();//关闭缓冲流
                    file.setLastModified(System.currentTimeMillis());
                } catch (IOException e) {
                    e.printStackTrace();
                    return false;
                }
            }
        }
        return true;
    }

    public String getFileName() {
        if (_file == null)
            return "";
        return _file.getAbsolutePath();
    }

    public String getAbsolutePath() {
        if (_file == null)
            return "";
        return _file.getAbsolutePath();
    }

    public static String getFileName(String pathAndName) {
        int start = pathAndName.lastIndexOf("/");
        int end = pathAndName.lastIndexOf(".");
        if (start != -1 && end != -1) {
            return pathAndName.substring(start + 1, end);
        } else {
            return "";
        }
    }

    public static String getFileNameAndType(String pathAndName) {
        int start = pathAndName.lastIndexOf("/");
        if (start != -1) {
            return pathAndName.substring(start + 1, pathAndName.length());
        } else {
            return "";
        }
    }

    public static String getFilePath(String pathAndName) {
        int start = pathAndName.lastIndexOf("/");
        if (start != -1 && start > 0) {
            return pathAndName.substring(0, start);
        } else {
            return "";
        }
    }

    /**
     * 删除文件夹内所有文件
     *
     * @param filePath       文件路径
     * @param deleteThisPath 是否删除改文件夹
     */
    public static void deleteFolderFile(String filePath, boolean deleteThisPath) {
        if (TextUtils.isEmpty(filePath)) {
            return;
        }
        try {
            File file = new File(filePath);
            if (file.isDirectory()) { //目录
                File files[] = file.listFiles();
                for (int i = 0; i < files.length; i++) {
                    deleteFolderFile(files[i].getAbsolutePath(), true);
                }
            }
            if (deleteThisPath) {
                if (!file.isDirectory()) { //如果是文件，删除
                    file.delete();
                } else { //目录
                    if (file.listFiles().length == 0) { //目录下没有文件或者目录，删除
                        file.delete();
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }/**
     * 获取不带扩展名的文件名
     *
     * @param filename 文件名
     * @return String 不带扩展名的文件名
     */
    public static String getFileNameNoExtension(String filename) {
        if ((filename.length() > 0)) {
            int dot = filename.lastIndexOf('.');
            if ((dot > -1) && (dot < (filename.length()))) {
                return filename.substring(0, dot);
            }
        }
        return filename;
    }

    /**
     * 获取文件的扩展名
     *
     * @param filename 文件的名
     * @return 文件的扩展名
     */
    public static String getFileNameExtension(String filename) {
        if ((filename.length() > 0)) {
            int dot = filename.lastIndexOf('.');
            if ((dot > -1) && (dot < (filename.length()))) {
                return filename.substring(dot + 1, filename.length());
            }
        }
        return "";
    }

    /**
     * 删除文件夹
     *
     * @param dir         文件夹
     * @param includeSelf 是否包括本身
     */
    public static void deleteDir(File dir, boolean includeSelf) {
        File[] files = dir.listFiles();
        if (files != null) {
            for (File f : files) {
                if (f.isDirectory()) {
                    deleteDir(f, true);
                } else {
                    f.delete();
                }
            }
        }
        if (includeSelf)
            dir.delete();
    }

}
