package hk.advanpro.android.core.utils;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.location.LocationManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.Uri;
import android.os.Environment;
import android.provider.Settings;

import java.io.File;

/**
 * Created by jalin on 2017/12/7.
 */

public class SystemUtils {

    public static String getExternalFilesDir(Context context) {
        File folder = context.getExternalFilesDir(null);
        if (folder == null) {
            File dir = Environment.getExternalStorageDirectory();
            folder = new File(dir + "/Android/data/" + context.getPackageName() + "/");
            if (!folder.exists()) folder.mkdir();
        }
        return folder.getAbsolutePath();
    }

    /**
     * 获取当前程序的包信息
     *
     * @return 当前程序的包信息
     */
    public static PackageInfo getApkPackInfo(Context context) {
        try {
            //获取packagemanager的实例
            PackageManager packageManager = context.getPackageManager();
            //getPackageName()是你当前类的包名，0代表是获取版本信息
            PackageInfo packInfo = packageManager.getPackageInfo(context.getPackageName(), 0);
            return packInfo;
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * gps是否可用
     *
     * @param context
     * @return
     */
    public static final boolean isGpsEnable(final Context context) {
        LocationManager locationManager
                = (LocationManager) context.getSystemService(Context.LOCATION_SERVICE);
        boolean gps = locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER);
        boolean network = locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER);
        if (gps || network) {
            return true;
        }
        return false;
    }

    public static final void gotoOpenGps(Activity activity, int requestCode) {
        //跳转到gps设置页
        Intent intent = new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        activity.startActivityForResult(intent, requestCode);
    }

    /**
     * 判断是否是当前是否WIFI
     */
    public static boolean isCurrentNetworkWIFI(Context context) {
        ConnectivityManager cm = (ConnectivityManager) context.getApplicationContext().getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo networkInfo = cm.getActiveNetworkInfo();
        return networkInfo != null && networkInfo.getType() == ConnectivityManager.TYPE_WIFI;
    }

    /**
     * 打开系统浏览器
     *
     * @param context
     * @param url
     */
    public static void openSystemWebView(Context context, String url) {
        Intent intent = new Intent();
        intent.setAction("android.intent.action.VIEW");
        Uri content_url = Uri.parse(url);
        intent.setData(content_url);
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        context.startActivity(intent);
    }
}
