package hk.advanpro.android.core.utils;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

/**
 * Created by jalin on 2017/9/19.
 */

public class ZipUtil {
    public static final String TAG = "ZipUtil";

    /**
     * 压缩数据
     * @param data byte array
     * @return 压缩后的byte array
     */
    public static byte[] compress(byte[] data) {
        GZIPOutputStream gzip = null;
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        try {
            gzip = new GZIPOutputStream(out);
            gzip.write(data);
            gzip.finish();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (gzip != null) gzip.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return out.toByteArray();
    }

    /**
     * 解压数据
     * @param data byte array
     * @return 解压数据后的 byte array
     */
    public static byte[] uncompress(byte[] data) {
        GZIPInputStream gzip = null;
        ByteArrayInputStream in = new ByteArrayInputStream(data);
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        try {
            gzip = new GZIPInputStream(in);
            while (gzip.available() > 0) {
                int b = gzip.read();
                out.write(b);
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (gzip != null) gzip.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return out.toByteArray();
    }

    /**
     *  压缩文件
     *
     * @param source 源文件
     * @param target 目标文件
     * @throws IOException IOException
     */
    public static void compressFile(File source, File target) throws IOException {
        FileInputStream fin = null;
        FileOutputStream fout = null;
        GZIPOutputStream gzout = null;
        try {
            fin = new FileInputStream(source);
            fout = new FileOutputStream(target);
            gzout = new GZIPOutputStream(fout);
            byte[] buf = new byte[1024];
            int num;
            while ((num = fin.read(buf)) != -1) {
                gzout.write(buf, 0, num);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (gzout != null)
                gzout.close();
            if (fout != null)
                fout.close();
            if (fin != null)
                fin.close();
        }
    }
}
